@@ -7,7 +7,7 @@ use base 'Module::Build';
 use lib "inc";
 use File::Spec::Functions qw(catfile rel2abs);
 use ExtUtils::Command;
-use LWP::Simple qw(getstore head);
+use File::Fetch;
 use File::Find;
 use File::Path qw();
 use File::ShareDir;
@@ -15,6 +15,7 @@ use File::Temp qw(tempdir tempfile);
 use Digest::SHA qw(sha1_hex);
 use Archive::Extract;
 use Config;
+use ExtUtils::Liblist;
 use Text::Patch;
 use IPC::Run3;
 
@@ -34,29 +35,21 @@ sub ACTION_install {
 sub ACTION_code {
   my $self = shift;
 
-  if ( ! -e 'build_done' ) {
+  unless (-e 'build_done') {
     my $inst = $self->notes('already_installed_lib');
     if (defined $inst) {
       $self->config_data('config', { LIBS   => $inst->{lflags},
                                      INC    => $inst->{cflags},
                                    });
     }
-    else {    
-      my $ans = $ENV{TRAVIS} ? 'n' : $self->prompt("\nWanna see more debug info during build (y/n)?", 'n');
-      $self->notes('build_debug_info', lc($ans) eq 'y' ? 1 : 0);
-
+    else {
       # important directories
       my $download = 'download';
       my $patches = 'patches';
       my $build_src = 'build_src';
       # we are deriving the subdir name from VERSION as we want to prevent
       # troubles when user reinstalls the newer version of Alien package
-      my $share_subdir = $self->{properties}->{dist_version};
-      if ($self->notes('is_devel_cvs_version')) {
-        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
-        $share_subdir .= sprintf("_CVS_%02d%02d%02d_%02d%02d",$year+1900-2000,$mon+1,$mday,$hour,$min);
-      }      
-      my $build_out = catfile('sharedir', $share_subdir);
+      my $build_out = catfile('sharedir', $self->{properties}->{dist_version});
       $self->add_to_cleanup($build_out);
 
       # store info into CofigData
@@ -66,64 +59,38 @@ sub ACTION_code {
 
       # prepare sources
       my $unpack;
-      $unpack = (-d "$build_src/iup") && !$ENV{TRAVIS} ? $self->prompt("\nDir '$build_src/iup' exists, wanna replace with clean sources?", "n") : 'y';
+      $unpack = (-d "$build_src/iup") ? $self->prompt("\nDir '$build_src/iup' exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
-        File::Path::rmtree("$build_src/iup") if -d "$build_src/iup";
         $self->prepare_sources($self->notes('iup_url'), $self->notes('iup_sha1'), $download, $build_src);
         if ($self->notes('iup_patches')) {
           $self->apply_patch("$build_src/iup", $_)  foreach (@{$self->notes('iup_patches')});
         }
       }
 
-      $unpack = (-d "$build_src/im") && !$ENV{TRAVIS} ? $self->prompt("\nDir '$build_src/im'  exists, wanna replace with clean sources?", "n") : 'y';
+      $unpack = (-d "$build_src/im") ? $self->prompt("\nDir '$build_src/im'  exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
-        File::Path::rmtree("$build_src/im") if -d "$build_src/im";
         $self->prepare_sources($self->notes('im_url'), $self->notes('im_sha1'), $download, $build_src);
         if ($self->notes('im_patches')) {
           $self->apply_patch("$build_src/im", $_)  foreach (@{$self->notes('im_patches')});
         }
       }
 
-      $unpack = (-d "$build_src/cd") && !$ENV{TRAVIS} ? $self->prompt("\nDir '$build_src/cd'  exists, wanna replace with clean sources?", "n") : 'y';
+      $unpack = (-d "$build_src/cd") ? $self->prompt("\nDir '$build_src/cd'  exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
-        File::Path::rmtree("$build_src/cd") if -d "$build_src/cd";
         $self->prepare_sources($self->notes('cd_url'), $self->notes('cd_sha1'), $download, $build_src);
         if ($self->notes('cd_patches')) {
           $self->apply_patch("$build_src/cd", $_)  foreach (@{$self->notes('cd_patches')});
         }
       }
-      
-      ### XXX hack for handling github tarballs
-      unless (-d "$build_src/cd" && -d "$build_src/im" && -d "$build_src/iup") {
-        for my $f (glob("$build_src/*")) {
-          if ($f =~ m!^\Q$build_src\E/.*?(im|cd|iup).*$!) {
-	    print "renaming: $f $build_src/$1\n";	   
-	    rename ($f, "$build_src/$1");
-	  }
-        }
-      }
 
-      my $m = $self->notes('build_debug_info') && !$ENV{TRAVIS} ? $self->prompt("\nDo you want to see all messages during 'make' (y/n)?", 'n') : 'n';
+      my $m = $self->prompt("\nDo you want to see all messages during 'make' (y/n)?", 'n');
       $self->notes('build_msgs', lc($m) eq 'y' ? 1 : 0);
-      
-      my $large_imglib = $ENV{TRAVIS} ? 'y' : lc($self->prompt("Do you wanna compile built-in images with large (48x48) size? ", "y"));
-      $self->notes('build_large_imglib', lc($large_imglib) eq 'y' ? 1 : 0);
 
       # go for build
-      my $success = $self->build_binaries($build_out, $build_src);
-      my $done    = $self->config_data('info_done');
-      my $iuplibs = $self->config_data('iup_libs');
-      if ($self->notes('build_debug_info')) {
-        print STDERR "Build result: $done->{$_} - $_\n" foreach (sort keys %$done);
-        print STDERR "Output libs : $iuplibs->{$_} - $_\n" foreach (sort keys %$iuplibs);
-      }
-      die "###BUILD FAILED### essential libs (iup/im/cd) not built!" unless $done->{"iup:iup"} && $done->{"iup:iupim"} && $done->{"iup:iupcd"};
-      die "###BUILD FAILED###" unless $success;
-      #DEBUG: die intentionally at this point if you want to see build details from cpan testers  
-      print STDERR "RESULT: OK!\n";
+      $self->build_binaries($build_out, $build_src);
 
-      # store info about build to ConfigData      
-      $self->config_data('share_subdir', $share_subdir);
+      # store info about build to ConfigData
+      $self->config_data('share_subdir', $self->{properties}->{dist_version});
       $self->config_data('config', { PREFIX => '@PrEfIx@',
                                      LIBS   => '-L' . $self->quote_literal('@PrEfIx@/lib') .
                                                ' -l' . join(' -l', @{$self->config_data('linker_libs')}) .
@@ -133,78 +100,44 @@ sub ACTION_code {
                                    });
     }
     # mark sucessfully finished build
-    $self->touchfile('build_done');
+    local @ARGV = ('build_done');
+    ExtUtils::Command::touch();
   }
   $self->SUPER::ACTION_code;
 }
 
 sub prepare_sources {
-  my ($self, $url, $sha1, $download, $build_src) = @_;  
-  my $archive = $self->fetch_file( url=>$url, sha1=>$sha1, localdir=>$download );
-  #XXX hack
-  if ($archive !~ /\.(tar.gz|tgz|tar|zip|tbz|tar.bz2)$/) {
-    rename($archive, "$archive.$$.tgz");
-    $archive = "$archive.$$.tgz";
-  }
-  my $ae = Archive::Extract->new( archive => $archive );
-  die "###ERROR### Cannot extract tarball ", $ae->error unless $ae->extract(to => $build_src);
+  my ($self, $url, $sha1, $download, $build_src) = @_;
+  $self->fetch_file($url, $sha1, $download);
+   my $archive = catfile($download, File::Fetch->new(uri => $url)->file);
+   my $ae = Archive::Extract->new( archive => $archive );
+   die "###ERROR### Cannot extract tarball ", $ae->error unless $ae->extract(to => $build_src);
 }
 
 sub fetch_file {
-  my ($self, %args) = @_;
-  
-  my $url = $args{url};
-  my $sha1 = $args{sha1};
-  my $localdir = $args{localdir};
-  my $localfile = $args{localfile};
-  die "###ERROR### fetch_file: undefined url\n" unless $url;
-  
-  # create $localdir if necessary
-  File::Path::mkpath($localdir) unless $localdir && -d $localdir;
-  
-  # handle redirects
-  my $head = head($url);
-  $url = $head->request->uri if defined $head;  
-  
-  # download destination
-  unless ($localfile) {
-   $localfile = $url;
-   $localfile =~ s/^.*?([^\\\/]+)$/$1/; #skip all but file part of URL
-   $localfile =~ s/\?.*$//; #skip URL params
+  my ($self, $url, $sha1sum, $download) = @_;
+  die "###ERROR### fetch_file() undefined url\n" unless $url;
+  die "###ERROR### fetch_file() undefined sha1sum\n" unless $sha1sum;
+  my $ff = File::Fetch->new(uri => $url);
+  my $fn = catfile($download, $ff->file);
+  if (-e $fn) {
+    print STDERR "Checking checksum for already existing '$fn'...\n";
+    return 1 if $self->check_sha1sum($fn, $sha1sum);
+    unlink $fn; #exists but wrong checksum
   }
-  $localfile = File::Spec->catfile($localdir, $localfile) if $localdir;
-  
-  # check existing file
-  if (-f $localfile) {
-    if ($sha1) {
-      if ($self->check_sha1sum($localfile, $sha1)) {
-	return rel2abs($localfile);
-      }
-      else {
-	warn "Checksum FAILURE";	
-      }
-    }
-    unlink $localfile; # if sha1 not given we force re-download
+  print STDERR "Fetching '$url'...\n";
+  my $fullpath = $ff->fetch(to => $download);
+  die "###ERROR### Unable to fetch '$url'" unless $fullpath;
+  if (-e $fn) {
+    print STDERR "Checking checksum for '$fn'...\n";
+    return 1 if $self->check_sha1sum($fn, $sha1sum);
+    die "###ERROR### Checksum failed '$fn'";
   }
-    
-  # download  
-  warn "Fetching '$url'...\n";
-  my $rv = getstore($url, $localfile);
-  die "###ERROR### fetch_file: download error - return code '$rv'\n" unless $rv == 200;
-  die "###ERROR### fetch_file: download error - '$localfile' not saved\n" unless -f $localfile;
-  
-  # checksum
-  if ($sha1) {
-    die "###ERROR### fetch_file: checksum failed" unless $self->check_sha1sum($localfile, $sha1);
-  }
-  
-  return rel2abs($localfile);
+  die "###ERROR### fetch_file() failed '$fn'";
 }
 
 sub check_sha1sum {
   my ($self, $file, $sha1sum) = @_;
-  return 1 if $sha1sum eq 'DO_NOT_CHECK_SHA1';
-  warn "Checking checksum for '$file'...\n";
   my $sha1 = Digest::SHA->new;
   my $fh;
   open($fh, $file) or die "###ERROR## Cannot check checksum for '$file'\n";
@@ -231,12 +164,6 @@ sub quote_literal {
 
 sub check_installed_lib {
   my ($self) = @_;
-  
-  #xxxTODO
-  #we not only need to detect the presence we also need to exactly know what libs are there - necessary for havelib() function
-  print STDERR "\nNOTICE:\nDetection of preinstalled iup+cd+im is disabled since v0.115!\nPlease contact the module author if you are missing this feature.\n\n";
-  return 0;
-  
   my $idir = $ENV{IUP_DIR} || '';
   my @candidates;
   push(@candidates, { L => "$idir/lib", I => "$idir/include" }) if -d $idir;
@@ -285,8 +212,8 @@ sub check_header {
   $cflags ||= '';
   my @header = ref($h) ? @$h : ( $h );
 
-  my ($fs, $src) = tempfile('tmpfileXXXXXX', SUFFIX => '.c', UNLINK => 1);
-  my ($fo, $obj) = tempfile('tmpfileXXXXXX', SUFFIX => '.o', UNLINK => 1);
+  my ($fs, $src) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.c', UNLINK => 1);
+  my ($fo, $obj) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.o', UNLINK => 1);
   my $inc = '';
   $inc .= "#include <$_>\n" foreach @header;
   syswrite($fs, <<MARKER); # write test source code
@@ -295,16 +222,20 @@ int demofunc(void) { return 0; }
 
 MARKER
   close($fs);
+  local *OLDERR;
+  open OLDERR, ">&", STDERR;
+  open STDERR, ">", File::Spec->devnull();
   $src = $self->quote_literal($src);
   $obj = $self->quote_literal($obj);
   #Note: $Config{cc} might contain e.g. 'ccache cc' (FreeBSD 8.0)
-  my $rv = run3("$Config{cc} -c -o $obj $src $cflags", \undef, \undef, \undef, { return_if_system_error => 1 } );
-  return ($rv == 1 && $? == 0) ? 1 : 0;
+  my $rv = system("$Config{cc} -c -o $obj $src $cflags");
+  open(STDERR, ">&", OLDERR);
+  return ($rv == 0) ? 1 : 0;
 }
 
 # check presence of lib(s) specified as params
 sub check_lib {
-  my ($self, $l, $cflags, $lflags) = @_;
+  my ($self, $l, $cflags, $lflags, $output) = @_;
   $cflags ||= '';
   $lflags ||= '';
   $cflags =~ s/[\r\n]//g;
@@ -312,9 +243,9 @@ sub check_lib {
   my @libs = ref($l) ? @$l : ( $l );
   my $liblist = scalar(@libs) ? '-l' . join(' -l', @libs) : '';
 
-  my ($fs, $src) = tempfile('tmpfileXXXXXX', SUFFIX => '.c', UNLINK => 1);
-  my ($fo, $obj) = tempfile('tmpfileXXXXXX', SUFFIX => '.o', UNLINK => 1);
-  my ($fe, $exe) = tempfile('tmpfileXXXXXX', SUFFIX => '.out', UNLINK => 1);
+  my ($fs, $src) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.c', UNLINK => 1);
+  my ($fo, $obj) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.o', UNLINK => 1);
+  my ($fe, $exe) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.out', UNLINK => 1);
   syswrite($fs, <<MARKER); # write test source code
 int main() { return 0; }
 
@@ -322,13 +253,12 @@ MARKER
   close($fs);
   $src = $self->quote_literal($src);
   $obj = $self->quote_literal($obj);
-  $exe = $self->quote_literal($exe);
-  my $output;
+  $exe = $self->quote_literal($exe);    
   #Note: $Config{cc} might contain e.g. 'ccache cc' (FreeBSD 8.0)
-  my $rv1 = run3("$Config{cc} -c -o $obj $src $cflags", \undef, \$output, \$output, { return_if_system_error => 1 } );
+  my $rv1 = run3("$Config{cc} -c -o $obj $src $cflags", \undef, \$output, \$output, { return_if_system_error => 1 } );  
   unless ($rv1 == 1 && $? == 0) {
     #print STDERR "OUTPUT(compile):\n$output\n" if $output;
-    return 0
+    return 0 
   }
   my $rv2 = run3("$Config{ld} $obj -o $exe $lflags $liblist", \undef, \$output, \$output, { return_if_system_error => 1 } );
   unless ($rv2 == 1 && $? == 0) {
@@ -358,7 +288,7 @@ sub apply_patch {
     $k =~ s|\\|/|g;
     $k =~ s|^[^/]*/(.*)$|$1|;
     $k = catfile($dir_to_be_patched, $k);
-    print STDERR "- gonna patch '$k'\n" if $self->notes('build_debug_info');
+    print STDERR "Gonna patch file '$k'\n" if $self->notes('build_debug_info');
 
     open(SRC, $k) or die "###ERROR### Cannot open file: '$k'\n";
     $src  = <SRC>;
@@ -381,7 +311,7 @@ sub run_output_tail {
   my ($self, $limit, @cmd) = @_;
   my $output;
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
-  print STDERR "- running (stdout+stderr redirected)...\n";
+  print STDERR "Running (stdout+stderr redirected)...\n";
   my $rv = run3(\@cmd, \undef, \$output, \$output, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
   $output = substr $output, -$limit if defined $limit; # we want just last N chars
@@ -398,11 +328,11 @@ sub run_output_on_error {
   my ($self, $limit, @cmd) = @_;
   my $output;
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
-  print STDERR "- running (stdout+stderr redirected)...\n";
+  print STDERR "Running (stdout+stderr redirected)...\n";
   my $rv = run3(\@cmd, \undef, \$output, \$output, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
   if ($success) {
-    print STDERR "- finished successfully (output suppressed)\n";
+    print STDERR "Finished successfully (output suppressed)\n";
   }
   else {
     $output = substr $output, -$limit if defined $limit; # we want just last N chars
@@ -421,7 +351,7 @@ sub run_output_std {
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
   my $rv = run3(\@cmd, undef, undef, undef, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
-  print STDERR "- finished successfully\n" if ($success);
+  print STDERR "Finished successfully\n" if ($success);
   return $success;
 }
 
@@ -467,7 +397,7 @@ sub find_file {
 
 sub sort_libs {
   my ($self, @unsorted) = @_;
-  my @wanted_order = qw/iupcontrols iup_pplot iupcd iupgl iupim iupimglib iupole iupweb iuptuio iupwin iupmot iupgtk iup cdgl cdpdf cdwin cdx11 cdgdk cd freetype6 freetype freetype-6 ftgl pdflib im_fftw im_jp2 im_process im/;
+  my @wanted_order = qw/iupwin iupmot iupgtk iup iupcontrols iup_pplot iupcd iupgl iupim iupimglib iupole iupweb iuptuio cdwin cdx11 cdgdk cd cdgl cdpdf freetype6 freetype freetype-6 ftgl pdflib im im_fftw im_jp2 im_process/;
   my @sorted;
   my %u;
 
@@ -487,14 +417,5 @@ sub sort_libs {
   return @sorted;
 }
 
-sub touchfile {
-  my $self = shift;
-  my $t    = time;
-  foreach my $file (@_) {
-    open(FILE,">>$file") || die "Cannot write $file:$!";
-    close(FILE);
-    utime($t,$t,$file);
-  }
-}
 
 1;
